/*
 * MSC (MMC/SD controller) structures.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <stdint.h>



/* MMC/SD structures. */

struct CID
{
  uint16_t month:4, year:8, reserved:4;
  uint32_t serial;
  uint8_t revision;
  char name[5];
  uint16_t oem;
  uint8_t manufacturer;
} __attribute__((packed));

struct CSD
{
  uint8_t  reserved0:2,
           format:2,
           temp_write_prot:1,
           perm_write_prot:1,
           copy:1,
           format_group:1;
  uint16_t reserved1:5,
           write_block_partial:1,
           write_blocklen:4,
           write_time_factor:3,
           reserved2:2,
           write_prot_group_enable:1;
  uint64_t write_prot_group_size:7,
           erase_sector_size:7,
           erase_single_block_enable:1,
           device_size_multiplier:3,
           max_write_current_max:3,
           max_write_current_min:3,
           max_read_current_max:3,
           max_read_current_min:3,
           device_size:12,
           reserved3:2,
           dsr_implemented:1,
           read_block_misalign:1,
           write_block_misalign:1,
           read_block_partial:1,
           read_blocklen:4,
           card_command_classes:12;
  uint8_t  tran_speed,
           data_read_access_time_2,
           data_read_access_time_1,
           reserved4:6,
           csd:2;
} __attribute__((packed));



/* Generic card structure. */

struct msc_card
{
  uint16_t rca;
  uint32_t ocr;
  uint8_t bus_width;
  struct CID cid;
  struct CSD csd;
};
